package App::SeismicUnixGui;

$VERSION = '0.82.1';
use strict;
use warnings;

=head1 NAME

App::SeismicUnixGui - A graphical user interface for Seismic Unix

=head1 DESCRIPTION

SeismicUnixGui (alpha) is a graphical user interface (GUI) 
to select parameters for Seismic Unix (SU) modules. 
Seismic Unix (Stockwell, 1999) is a widely distributed 
free software package for processing seismic reflection 
and signal processing. 
Perl/Tk is a mature, well-documented and free, 
object-oriented graphical user interface for Perl. 

1;
